@@ -34,6 +34,7 @@ class User(Record):
         self.format = 'json'
         self._fees = None
         self._loans = None
+        self._requests = None
 
     def __repr__(self) -> str:
         """Get a string representation of the object. Useful for logs.
@@ -102,6 +103,29 @@ class User(Record):
         else:
             self._handle_error(r, f'unable to fetch user loans')
 
+    def _fetch_requests(self) -> Optional[List['users.Request']]:
+        """Fetch request data of the current user
+
+        :return: list of :class:`almapiwrapper.users.Request` objects"""
+
+        r = self._api_call('get',
+                           f'{self.api_base_url}/{self.primary_id}/requests',
+                           params={'limit': '100'},
+                           headers=self._get_headers())
+        if r.ok is True:
+
+            logging.info(f'{repr(self)}: requests data available')
+            request_data = r.json()
+            if 'user_request' not in request_data or request_data['user_request'] is None:
+                logging.warning(f'{repr(self)}: no request in the account')
+                return []
+            user_requests = []
+            for request_data in request_data['user_request']:
+                user_requests.append(users.Request(user=self, data=JsonData(request_data)))
+            return user_requests
+        else:
+            self._handle_error(r, f'unable to fetch user requests')
+
     def save(self) -> 'User':
         """save() -> 'User'
         Save a user record in the 'records' folder
@@ -177,17 +201,17 @@ class User(Record):
 
         :return: primary ID of the user
         """
-        if self._data is not None:
-            return self.data['primary_id']
-        else:
-            return self._primary_id
+
+        return self._primary_id
 
     @primary_id.setter
     def primary_id(self, primary_id) -> None:
         """Set the primary ID
 
         If json data is available, change the primary ID
-        at the data level.
+        at the data level. To modify the primary ID of the user,
+        do not use this method property but change directly the primary_id
+        in the data attribute.
 
         :param primary_id: new primary ID to set
 
@@ -217,6 +241,16 @@ class User(Record):
 
         return self._loans
 
+    @property
+    def requests(self) -> Optional[List['users.Request']]:
+        """Property returning the list of the requests of the user
+
+        :return: list of :class:`almapiwrapper.users.Request` objects"""
+        if self._requests is None:
+            self._requests = self._fetch_requests()
+
+        return self._requests
+
     @check_error
     def create_loan(self,
                     library: str,
@@ -345,6 +379,41 @@ class User(Record):
             logging.warning(f'{repr(self)}: NO synchronization test note exists')
         return has_synchro_note
 
+    @check_error
+    def dedup_notes(self) -> 'User':
+        """dedup_notes(self) -> 'User'
+        Remove all duplicated notes
+
+        Text of note is used to compare notes.
+        If the text is the same, the external note is preferred.
+
+        :return: object :class:`almapiwrapper.users.User`
+        """
+        notes_to_be_kept = []
+        for note in self.data['user_note']:
+            found = False
+            for note_to_be_kept in notes_to_be_kept:
+                if note_to_be_kept['note_text'] == note['note_text']:
+                    found = True
+
+                    # Prefer external note
+                    if note['segment_type'] == 'External' and note_to_be_kept['segment_type'] == 'Internal':
+                        # remove internal note and add the external one to be kept
+                        notes_to_be_kept.remove(note_to_be_kept)
+                        notes_to_be_kept.append(note)
+                    break
+
+            # Add the current tested note if no corresponding note is found
+            if found is False:
+                notes_to_be_kept.append(note)
+        self.data['user_note'] = notes_to_be_kept
+        return self
+
+    def fetch_requests(self) -> Optional[List['users.Request']]:
+        """fetch_requests(self) -> Optional[List['users.Request']]
+        Fetch the list of the requests of the user"""
+
+        pass
 
 class NewUser(User):
     """Class used to create new users
@@ -354,10 +423,14 @@ class NewUser(User):
     :var data: initial value: :class:`almapiwrapper.record.JsonData` of the user to create
     """
 
-    def __init__(self, zone: str, env: Literal['P', 'S'], data: JsonData):
+    def __init__(self, zone: str, env: Literal['P', 'S'], data: Union[JsonData, dict]):
         """Constructor to create new users
         """
+        if isinstance(data, dict):
+            data = JsonData(data)
+
         super(User, self).__init__(zone, env, data)
+        self._primary_id = self.data['primary_id']
         self.area = 'Users'
         self.format = 'json'
 
